package com.milieukontakt.reagiram.adapters;

import android.content.Context;
import android.text.Html;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import com.milieukontakt.reagiram.R;
import com.milieukontakt.reagiram.models.RssItem;
import com.squareup.picasso.Picasso;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;


public class RssAdapter extends BaseAdapter {

    // a little magic to shorten the date that is received from the feed
    // (removes time of day and time zone)
    SimpleDateFormat beforeFormat;
    SimpleDateFormat afterFormat;
    private Context mContext = null;
    private ArrayList<RssItem> mItems = null;

    // constructor
    public RssAdapter(Context context, ArrayList<RssItem> items) {
        this.mContext = context;
        this.mItems = items;

        // a little magic to shorten the date that is received from the feed
        // (removes time of day and time zone)
        beforeFormat = new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss Z");
        afterFormat = new SimpleDateFormat("d MMM yyyy");
    }

    // get the number of items present in the list adapter
    public int getCount() {
        return mItems.size();
    }

    // get specific object (we will use this when an item is clicked and we want
    // to display more info by opening the link provided in the object)
    public RssItem getItem(int position) {
        return mItems.get(position);
    }

    public long getItemId(int position) {
        return 0;
    }

    public View getView(int position, View convertView, ViewGroup parent) {
        // get the specific list entry
        final RssItem object = mItems.get(position);
        // make custom list entries specified by the apropriate layout
        if (convertView == null) {
            LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            convertView = inflater.inflate(R.layout.item_rss, null);
            // use the view holder pattern
            ViewHolder viewHolder = new ViewHolder();
            viewHolder.title = (TextView) convertView.findViewById(R.id.txt_title);
            viewHolder.date = (TextView) convertView.findViewById(R.id.txt_date);
            viewHolder.description = (TextView) convertView.findViewById(R.id.txt_description);
            viewHolder.snippet = (ImageView) convertView.findViewById(R.id.img_snippet);
            convertView.setTag(viewHolder);

        }

        // get the view holder
        ViewHolder holder = (ViewHolder) convertView.getTag();
        holder.title.setText(Html.fromHtml(object.getTitle()).toString());
        try {
            holder.date.setText(afterFormat.format(beforeFormat.parse(object.getPubDate())));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        holder.description.setText(Html.fromHtml(object.getDescription()).toString());
        Picasso.with(mContext)
                .load(object.getSnippetUrl())
                .placeholder(R.drawable.ic_placeholder)
                .error(R.drawable.ic_placeholder)
                .into(holder.snippet);

        // return the inflated view with the added info
        return convertView;
    }

    static class ViewHolder {
        TextView title;
        TextView date;
        TextView description;
        ImageView snippet;
    }
}
