package com.milieukontakt.reagiram.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

import com.google.gson.Gson;
import com.milieukontakt.reagiram.R;
import com.milieukontakt.reagiram.models.Profile;
import com.milieukontakt.reagiram.models.Statuses;
import com.milieukontakt.reagiram.utils.Consts;
import com.milieukontakt.reagiram.utils.Utils;

import java.util.HashMap;
import java.util.List;


public class StatusesAdapter extends BaseAdapter {

    private Context mContext = null;
    private List<Statuses> mItems = null;
    private HashMap<Integer, String> statusMap;
    private String mId;

    // constructor
    public StatusesAdapter(Context context, List<Statuses> items) {
        this.mContext = context;
        this.mItems = items;

        statusMap = new HashMap<Integer, String>();
        statusMap.put(0, context.getString(R.string.hint_no_reports));
        statusMap.put(1, context.getString(R.string.status_accepted));
        statusMap.put(2, context.getString(R.string.status_pending));
        statusMap.put(3, context.getString(R.string.status_resolving));
        statusMap.put(4, context.getString(R.string.status_solved));

        String jsonProfile = Utils.getStringFromPrefs(context, Consts.PREF_KEY_PROFILE);
        if (jsonProfile != null)
            mId = new Gson().fromJson(jsonProfile, Profile.class).getId();
        if(mId == null)
            mId = "";
    }

    // get the number of items present in the list adapter
    public int getCount() {
        return mItems.size();
    }

    // get specific object (we will use this when an item is clicked and we want
    // to display more info by opening the link provided in the object)
    public Statuses getItem(int position) {
        return mItems.get(position);
    }

    public long getItemId(int position) {
        return 0;
    }

    public View getView(int position, View convertView, ViewGroup parent) {
        // get the specific list entry
        Statuses object = mItems.get(position);
        // make custom list entries specified by the apropriate layout
        if (convertView == null) {
            LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            convertView = inflater.inflate(R.layout.item_status, null);
            // use the view holder pattern
            ViewHolder viewHolder = new ViewHolder();
            viewHolder.addressAndTime = (TextView) convertView.findViewById(R.id.txt_address);
            viewHolder.status = (TextView) convertView.findViewById(R.id.txt_status);
            viewHolder.description = (TextView) convertView.findViewById(R.id.txt_description);
            convertView.setTag(viewHolder);

        }

        if(mId.equalsIgnoreCase(object.getUserId()))
            convertView.setBackgroundColor(mContext.getResources().getColor(R.color.trademark_green_transparent));
        else
            convertView.setBackgroundColor(mContext.getResources().getColor(R.color.card_white));

        // get the view holder
        ViewHolder holder = (ViewHolder) convertView.getTag();
        holder.addressAndTime.setText("");
        if (object.getTime() != null)
            holder.addressAndTime.append(object.getTime());
        if(object.getLocation() != null)
            holder.addressAndTime.append(", " + object.getLocation());
        if(holder.addressAndTime.getText().length() == 0)
            holder.addressAndTime.setVisibility(View.GONE);
        holder.status.setText(statusMap.get(Integer.valueOf(object.getStatus())));
        holder.description.setText(object.getDescription());
        // return the inflated view with the added info
        return convertView;
    }

    static class ViewHolder {
        TextView addressAndTime;
        TextView status;
        TextView description;
    }
}
