package com.milieukontakt.reagiram.fragments;

import android.animation.Animator;
import android.animation.ObjectAnimator;
import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.support.v4.app.Fragment;
import android.util.Xml;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AbsListView;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.milieukontakt.reagiram.R;
import com.milieukontakt.reagiram.ReportActivity;
import com.milieukontakt.reagiram.adapters.RssAdapter;
import com.milieukontakt.reagiram.models.RssItem;
import com.milieukontakt.reagiram.utils.Consts;
import com.milieukontakt.reagiram.utils.RSSParser;
import com.milieukontakt.reagiram.utils.Utils;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.xmlpull.v1.XmlPullParser;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

/**
 * Created by MedoHome on 11.10.13.
 */
public class HomeFragment extends Fragment implements View.OnClickListener {

    private View mView;
    private View mFooter;
    private ListView mList;
    private int lastFirstChild;
    private int lastY;
    private int state = Consts.ON_SCREEN;
    private ObjectAnimator hideAnimator;
    private ObjectAnimator returnAnimator;
    private String mCurrentPhotoPath;

    @SuppressLint("NewApi")
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        mView = inflater.inflate(R.layout.fragment_home, container, false);

        mFooter = mView.findViewById(R.id.footer_quick);
        mFooter.findViewById(R.id.txt_photo).setOnClickListener(this);
        mFooter.findViewById(R.id.txt_gallery).setOnClickListener(this);

        mList = (ListView) mView.findViewById(R.id.list_home);
        // do quick return for newer API version
        if (Build.VERSION.SDK_INT >= 14) {

            returnAnimator = ObjectAnimator.ofFloat(mFooter, "translationY", 0);
            returnAnimator.addListener(
                    new Animator.AnimatorListener() {
                        @Override
                        public void onAnimationStart(Animator animator) {
                            state = Consts.RETURNING;
                        }

                        @Override
                        public void onAnimationEnd(Animator animator) {
                            state = Consts.ON_SCREEN;
                        }

                        @Override
                        public void onAnimationCancel(Animator animator) {
                            state = Consts.OFF_SCREEN;
                        }

                        @Override
                        public void onAnimationRepeat(Animator animator) {
                        }
                    }
            );
            hideAnimator = ObjectAnimator.ofFloat(mFooter, "translationY", getResources().getDimension(R.dimen.footer_height));
            hideAnimator.addListener(
                    new Animator.AnimatorListener() {
                        @Override
                        public void onAnimationStart(Animator animator) {
                            state = Consts.HIDING;
                        }

                        @Override
                        public void onAnimationEnd(Animator animator) {
                            state = Consts.OFF_SCREEN;
                        }

                        @Override
                        public void onAnimationCancel(Animator animator) {
                            state = Consts.ON_SCREEN;
                        }

                        @Override
                        public void onAnimationRepeat(Animator animator) {
                        }
                    }
            );

            mList.setOnScrollListener(new AbsListView.OnScrollListener() {
                @Override
                public void onScrollStateChanged(AbsListView view, int scrollState) {

                }

                @Override
                public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount, int totalItemCount) {
                    // Computes the scroll Y position
                    final View firstChild = mList.getChildAt(firstVisibleItem);
                    int yPosition = firstChild == null ? 0 : firstChild.getTop();

                    switch (state) {
                        case Consts.OFF_SCREEN:
                            // Return quick return bar if first visible child is the same AND it's Y position increases
                            // Return quick return bar if first visible child changes to a previous sibling
                            // AND only if the quick return bar isn't already returning
                            if (!quickReturnBarIsReturning()
                                    && (firstVisibleItem == lastFirstChild && yPosition > lastY)
                                    || firstVisibleItem < lastFirstChild)
                                returnAnimator.start();
                            break;

                        case Consts.ON_SCREEN:
                            // Hide quick return bar if first visible child is the same AND it's Y position decreases
                            // Hide quick return bar if first visible child changes to a later sibling
                            // AND only if the quick return bar isn't already going away
                            if (!quickReturnBarIsGoingAway()
                                    && (firstVisibleItem == lastFirstChild && yPosition < lastY)
                                    || firstVisibleItem > lastFirstChild)
                                hideAnimator.start();
                            break;

                        case Consts.RETURNING:
                            // Cancel return of quick return bar if first visible child is the same AND it's Y position decreases
                            // Cancel return of quick return bar if first visible child changes to a later sibling
                            if ((firstVisibleItem == lastFirstChild && yPosition < lastY)
                                    || firstVisibleItem > lastFirstChild) {
                                returnAnimator.cancel();
                                hideAnimator.start();
                            }
                            break;

                        case Consts.HIDING:
                            // Cancel hide of quick return bar if first visible child is the same AND it's Y position increases
                            // Cancel hide of quick return bar if first visible child changes to a previous sibling
                            if ((firstVisibleItem == lastFirstChild && yPosition > lastY)
                                    || firstVisibleItem < lastFirstChild) {
                                hideAnimator.cancel();
                                returnAnimator.start();
                            }
                            break;
                    }
                    lastFirstChild = firstVisibleItem;
                    lastY = yPosition;
                }
            });
        }
        return mView;
    }

    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
    private boolean quickReturnBarIsReturning() {
        return returnAnimator.isRunning() || returnAnimator.isStarted();
    }

    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
    private boolean quickReturnBarIsGoingAway() {
        return hideAnimator.isRunning() || hideAnimator.isStarted();
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        long lastFeedTimestamp = Utils.getLongFromPrefs(getActivity(), Consts.PREF_KEY_RSS_TIMESTAMP);
        if (System.currentTimeMillis() - lastFeedTimestamp > Consts.RSS_VALIDITY) {
            // get new feed
            GetRSSFeed task = new GetRSSFeed();
            task.execute();
        } else {
            // parse saved feed
            String jsonFeed = Utils.getStringFromPrefs(getActivity(), Consts.PREF_KEY_RSS);
            ArrayList<RssItem> rssFeed = new Gson().fromJson(jsonFeed, new TypeToken<ArrayList<RssItem>>() {
            }.getType());
            // show the feed
            mList.addFooterView(getActivity().getLayoutInflater().inflate(R.layout.footer_trademark, null, false));
            mList.setAdapter(new RssAdapter(getActivity(), rssFeed));
            mList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                    if (position == parent.getCount() - 1)
                        return;
                    RssItem item = (RssItem) parent.getAdapter().getItem(position);
                    Intent browserIntent = new Intent(Intent.ACTION_VIEW, Uri.parse(item.getLink()));
                    startActivity(browserIntent);
                }
            });
        }
        super.onViewCreated(view, savedInstanceState);
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.txt_gallery:
                Intent intent = new Intent();
                intent.setType("image/*");
                intent.setAction(Intent.ACTION_GET_CONTENT);
                startActivityForResult(intent, Consts.ACTION_PICK_PHOTO);
                break;
            case R.id.txt_photo:
                if (Utils.isIntentAvailable(getActivity(), MediaStore.ACTION_IMAGE_CAPTURE)) {
                    Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
                    try {
                        File f = createImageFile();
                        takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, Uri.fromFile(f));
                        startActivityForResult(takePictureIntent, Consts.ACTION_TAKE_PHOTO);
                    } catch (Exception e) {
                        // TODO handle error
                        e.printStackTrace();
                    }
                }
                break;
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {

        if (requestCode == Consts.ACTION_PICK_PHOTO && data != null && data.getData() != null) {
            Uri _uri = data.getData();

            // image selected
            Cursor cursor = getActivity()
                    .getContentResolver()
                    .query(_uri, new String[]{android.provider.MediaStore.Images.ImageColumns.DATA}, null, null, null);
            cursor.moveToFirst();

            // get the image path
            String imageFilePath = cursor.getString(0);
            cursor.close();

            Intent reportActivity = new Intent(getActivity(), ReportActivity.class);
            reportActivity.putExtra(Consts.EXTRA_PHOTO_PATH, imageFilePath);
            startActivity(reportActivity);
        } else if (requestCode == Consts.ACTION_TAKE_PHOTO) {
            Intent reportActivity = new Intent(getActivity(), ReportActivity.class);
            reportActivity.putExtra(Consts.EXTRA_PHOTO_PATH, mCurrentPhotoPath);
            startActivity(reportActivity);
            galleryAddPic();
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    private File createImageFile() throws IOException {
        // create album directory
        File storageDir = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES), getString(R.string.app_name));
        if (!storageDir.exists())
            storageDir.mkdirs();
        // Create an image file name
        String imageFileName = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
        File image = File.createTempFile(
                imageFileName,
                ".jpg",
                storageDir
        );
        mCurrentPhotoPath = image.getAbsolutePath();
        return image;
    }

    private void galleryAddPic() {
        Intent mediaScanIntent = new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE);
        try {
            File f = new File(mCurrentPhotoPath);
            Uri contentUri = Uri.fromFile(f);
            mediaScanIntent.setData(contentUri);
            getActivity().sendBroadcast(mediaScanIntent);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private class GetRSSFeed extends AsyncTask<Void, Void, ArrayList<RssItem>> {

        @Override
        protected void onPreExecute() {
            if (!isAdded())
                return;
            getActivity().setProgressBarIndeterminateVisibility(true);
        }

        @Override
        protected ArrayList<RssItem> doInBackground(Void... params) {
            try {
                // defaultHttpClient
                DefaultHttpClient httpClient = new DefaultHttpClient();
                HttpPost httpPost = new HttpPost(Consts.RSS_URL);

                HttpResponse httpResponse = httpClient.execute(httpPost);
                HttpEntity httpEntity = httpResponse.getEntity();

                // parser initialization
                XmlPullParser parser = Xml.newPullParser();
                parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
                parser.setInput(httpEntity.getContent(), "UTF-8");
                // skip the first two tags so the parser is positioned at the
                // <channel> tag
                parser.nextTag();
                parser.nextTag();
                RSSParser customParser = new RSSParser();
                return customParser.readFeed(parser);
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }

        @Override
        protected void onPostExecute(ArrayList<RssItem> rssFeed) {
            if (!isAdded())
                return;
            // hide the progress bar
            getActivity().setProgressBarIndeterminateVisibility(false);
            // handle empty or null list
            if (rssFeed == null || rssFeed.isEmpty()) {
                if (!Utils.isConnected(getActivity()))
                    Toast.makeText(getActivity(), R.string.toast_no_connection, Toast.LENGTH_SHORT).show();
                else
                    Toast.makeText(getActivity(), R.string.toast_server_unavailable, Toast.LENGTH_SHORT).show();
                return;
            }
            // save to preferences
            String jsonFeed = new Gson().toJson(rssFeed);
            Utils.setStringToPrefs(getActivity(), Consts.PREF_KEY_RSS, jsonFeed);
            Utils.setLongToPrefs(getActivity(), Consts.PREF_KEY_RSS_TIMESTAMP, System.currentTimeMillis());

            // show the feed
            mList.addFooterView(getActivity().getLayoutInflater().inflate(R.layout.footer_trademark, null, false));
            mList.setAdapter(new RssAdapter(getActivity(), rssFeed));
            mList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                    if (position == parent.getCount() - 1)
                        return;
                    RssItem item = (RssItem) parent.getAdapter().getItem(position);
                    Intent browserIntent = new Intent(Intent.ACTION_VIEW, Uri.parse(item.getLink()));
                    startActivity(browserIntent);
                }
            });

        }
    }
}
