package com.milieukontakt.reagiram.fragments;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.Patterns;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.TextView;

import com.google.gson.Gson;
import com.milieukontakt.reagiram.R;
import com.milieukontakt.reagiram.asyncs.GetUserId;
import com.milieukontakt.reagiram.models.Profile;
import com.milieukontakt.reagiram.utils.Consts;
import com.milieukontakt.reagiram.utils.Utils;

/**
 * Created by MedoHome on 11.10.13.
 */
public class ProfileFragment extends Fragment implements View.OnClickListener {

    private View mView;
    private EditText mName, mLastName, mEmail;
    private RadioGroup mGender;
    private Spinner mAge;
    private TextView mSave;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        mView = inflater.inflate(R.layout.fragment_profile, container, false);

        mName = (EditText) mView.findViewById(R.id.edit_name);
        mLastName = (EditText) mView.findViewById(R.id.edit_last_name);
        mEmail = (EditText) mView.findViewById(R.id.edit_email);
        mGender = (RadioGroup) mView.findViewById(R.id.radio_gender);
        mAge = (Spinner) mView.findViewById(R.id.spinner_age);
        mSave = (TextView) mView.findViewById(R.id.txt_save);
        mSave.setOnClickListener(this);

        // check if we already have a profile
        String jsonProfile = Utils.getStringFromPrefs(getActivity(), Consts.PREF_KEY_PROFILE);
        if (jsonProfile != null)
            prefilValues(jsonProfile);

        return mView;
    }

    private void prefilValues(String jsonProfile) {
        Profile profile = new Gson().fromJson(jsonProfile, Profile.class);
        mName.setText(profile.getFirstName());
        mLastName.setText(profile.getLastName());
        mEmail.setText(profile.getEmail());

        String gender = profile.getGender();
        if (gender.equalsIgnoreCase(getString(R.string.hint_male)))
            ((RadioButton) mGender.findViewById(R.id.radio_male)).setChecked(true);
        else
            ((RadioButton) mGender.findViewById(R.id.radio_female)).setChecked(true);

        String[] age = getResources().getStringArray(R.array.age_items);
        for (int i = 0; i < age.length; i++)
            if (profile.getAge().equalsIgnoreCase(age[i]))
                mAge.setSelection(i);
    }

    @Override
    public void onClick(View v) {
        if (validate()) {
            // create the profile
            Profile newProfile = new Profile();
            newProfile.setFirstName(mName.getText().toString());
            newProfile.setLastName(mLastName.getText().toString());
            newProfile.setEmail(mEmail.getText().toString());
            RadioButton gender = (RadioButton) mGender.findViewById(mGender.getCheckedRadioButtonId());
            newProfile.setGender(gender.getText().toString());
            newProfile.setAge(mAge.getSelectedItem().toString());

            GetUserId task = new GetUserId();
            task.setmContext(getActivity());
            task.execute(newProfile);

            // return to report activity or finish it if coming from menu drawer
            if (getActivity().getSupportFragmentManager().getBackStackEntryCount() == 0)
                getActivity().finish();
            else
                getActivity().getSupportFragmentManager().popBackStack();
        }
    }

    private boolean validate() {
        if (mName.getText().toString().length() == 0) {
            mName.setError(getString(R.string.hint_required_field));
            mName.requestFocus();
            return false;
        }
        if (mLastName.getText().toString().length() == 0) {
            mLastName.setError(getString(R.string.hint_required_field));
            mLastName.requestFocus();
            return false;
        }
        if (mEmail.getText().toString().length() == 0) {
            mEmail.setError(getString(R.string.hint_required_field));
            mEmail.requestFocus();
            return false;
        }
        if (!mEmail.getText().toString().matches(Patterns.EMAIL_ADDRESS.pattern())) {
            mEmail.setError(getString(R.string.hint_invalid_email));
            mEmail.requestFocus();
            return false;
        }
        return true;
    }
}
