package com.milieukontakt.reagiram.fragments;

import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.telephony.TelephonyManager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.Spinner;
import android.widget.TextView;

import com.google.gson.Gson;
import com.milieukontakt.reagiram.R;
import com.milieukontakt.reagiram.models.Municipalities;
import com.milieukontakt.reagiram.models.Municipality;
import com.milieukontakt.reagiram.models.Profile;
import com.milieukontakt.reagiram.models.Report;
import com.milieukontakt.reagiram.models.ReportQueue;
import com.milieukontakt.reagiram.services.ReportService;
import com.milieukontakt.reagiram.utils.Consts;
import com.milieukontakt.reagiram.utils.Utils;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by MedoHome on 11.10.13.
 */
public class ReportFragment extends Fragment implements View.OnClickListener, LocationListener {

    private View mView;
    private Spinner mMunicipality, mCategory, mSubcategory;
    private EditText mLocation, mDescription;
    private TextView mSend, mSendWifi;
    private ImageView mSnippet;
    private String mCurrentPhotoPath;
    private String mLon;
    private String mLat;
    private LocationManager mLocationManager;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        mView = inflater.inflate(R.layout.fragment_report, container, false);

        // request location updates
        mLocationManager = (LocationManager) getActivity().getSystemService(Context.LOCATION_SERVICE);

        // Register the listener with the Location Manager to receive location updates
        mLocationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, this);
        mLocationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, this);

        mCurrentPhotoPath = getArguments().getString(Consts.EXTRA_PHOTO_PATH);

        mMunicipality = (Spinner) mView.findViewById(R.id.spinner_municipality);
        mCategory = (Spinner) mView.findViewById(R.id.spinner_category);
        mSubcategory = (Spinner) mView.findViewById(R.id.spinner_subcategory);
        mLocation = (EditText) mView.findViewById(R.id.edit_location);
        mDescription = (EditText) mView.findViewById(R.id.edit_description);
        mSend = (TextView) mView.findViewById(R.id.txt_send);
        mSendWifi = (TextView) mView.findViewById(R.id.txt_send_wifi);
        mSnippet = (ImageView) mView.findViewById(R.id.img_snippet);
        mSnippet.getViewTreeObserver().addOnGlobalLayoutListener(new ViewTreeObserver.OnGlobalLayoutListener() {
            @Override
            public void onGlobalLayout() {
                setSnippet();
                mSnippet.getViewTreeObserver().removeGlobalOnLayoutListener(this);
            }
        });

        mSend.setOnClickListener(this);
        mSendWifi.setOnClickListener(this);

        mCategory.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                ArrayAdapter<String> newAdapter = null;
                switch (position) {

                    case 0:
                        newAdapter = new ArrayAdapter<String>(getActivity(), android.R.layout.simple_spinner_dropdown_item,
                                getResources().getStringArray(R.array.subcategory_0_items));
                        break;
                    case 1:
                        newAdapter = new ArrayAdapter<String>(getActivity(), android.R.layout.simple_spinner_dropdown_item,
                                getResources().getStringArray(R.array.subcategory_1_items));
                        break;
                    case 2:
                        newAdapter = new ArrayAdapter<String>(getActivity(), android.R.layout.simple_spinner_dropdown_item,
                                getResources().getStringArray(R.array.subcategory_2_items));
                        break;
                    case 3:
                        newAdapter = new ArrayAdapter<String>(getActivity(), android.R.layout.simple_spinner_dropdown_item,
                                getResources().getStringArray(R.array.subcategory_3_items));
                        break;
                    case 4:
                        newAdapter = new ArrayAdapter<String>(getActivity(), android.R.layout.simple_spinner_dropdown_item,
                                getResources().getStringArray(R.array.subcategory_4_items));
                        break;
                }
                mSubcategory.setAdapter(newAdapter);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        // check connectivity
        // if not connected only report later should be visible
        if (!Utils.isConnected(getActivity()))
            mSend.setVisibility(View.GONE);
            // if connected via WiFi just send now should be visible
        else if (Utils.isConnectedWifi(getActivity())) {
            mSendWifi.setVisibility(View.GONE);
            mSend.setText(getString(R.string.btn_send));
        }
        // otherwise both buttons should be visible

        long lastMunicipalitiesTimestamp = Utils.getLongFromPrefs(getActivity(), Consts.PREF_KEY_MUNICIPALITIES_TIMESTAMP);
        String jsonMunicipalities = Utils.getStringFromPrefs(getActivity(), Consts.PREF_KEY_MUNICIPALITIES);
        // if we don't have a list of municipalities ping the service
        if (jsonMunicipalities != null) {
            Municipalities municipalities = new Gson().fromJson(jsonMunicipalities, Municipalities.class);
            String[] municipalitiesArray = new String[municipalities.getMunicipalities().size()];
            int i = 0;
            for (Municipality municipality : municipalities.getMunicipalities())
                municipalitiesArray[i++] = municipality.getName();

            mMunicipality.setAdapter(new ArrayAdapter<String>(getActivity(), android.R.layout.simple_spinner_dropdown_item, municipalitiesArray));
        } else if (System.currentTimeMillis() - lastMunicipalitiesTimestamp > Consts.MUNICIPALITIES_VALIDITY) {
            // get new list of municipalities if enough time has passed
            GetMunicipalities task = new GetMunicipalities();
            task.execute();
        }
        // else use the default values

        return mView;
    }

    @Override
    public void onPause() {
        mLocationManager.removeUpdates(this);
        super.onPause();
    }

    @Override
    public void onClick(View v) {
        // remove location updates
        mLocationManager.removeUpdates(this);
        // make new report
        Gson gson = new Gson();
        Report report = new Report();
        Profile profile = gson.fromJson(Utils.getStringFromPrefs(getActivity(), Consts.PREF_KEY_PROFILE), Profile.class);
        report.setProfile(profile);
        report.setCategory(mCategory.getSelectedItem().toString() + " / " + mSubcategory.getSelectedItem().toString());
        report.setDescription(mDescription.getText().toString());
        report.setLocation(mLocation.getText().toString());
        report.setMunicipality(mMunicipality.getSelectedItem().toString());
        report.setImageUrl(mCurrentPhotoPath);
        // set the operator
        TelephonyManager telephonyManager = ((TelephonyManager) getActivity().getSystemService(Context.TELEPHONY_SERVICE));
        if (telephonyManager != null && telephonyManager.getNetworkOperatorName() != null)
            report.setOperator(telephonyManager.getNetworkOperatorName());
        // set the time
        SimpleDateFormat sdf = new SimpleDateFormat(Consts.DATE_FORMAT);
        report.setTime(sdf.format(new Date()));
        // set the location
        report.setLongitude(mLon);
        report.setLatitude(mLat);

        // check if we have a queue
        String jsonQueue = Utils.getStringFromPrefs(getActivity(), Consts.PREF_KEY_QUEUE);
        ReportQueue queue = null;
        List<Report> reportList = null;
        // make new one if not existent
        if (jsonQueue == null) {
            queue = new ReportQueue();
            reportList = new ArrayList<Report>();
        }
        // append to existing if present
        else {
            queue = gson.fromJson(jsonQueue, ReportQueue.class);
            reportList = queue.getReports();
        }
        reportList.add(report);
        queue.setReports(reportList);
        // save queue to preferences
        Utils.setStringToPrefs(getActivity(), Consts.PREF_KEY_QUEUE, gson.toJson(queue, ReportQueue.class));

        switch (v.getId()) {
            case R.id.txt_send:
                getActivity().startService(new Intent(getActivity(), ReportService.class));
                break;
        }
        getActivity().finish();
    }

    private void setSnippet() {
        // Get the dimensions of the View
        int targetW = mSnippet.getWidth();
        int targetH = mSnippet.getHeight();

        // Get the dimensions of the bitmap
        BitmapFactory.Options bmOptions = new BitmapFactory.Options();
        bmOptions.inJustDecodeBounds = true;
        BitmapFactory.decodeFile(mCurrentPhotoPath, bmOptions);
        int photoW = bmOptions.outWidth;
        int photoH = bmOptions.outHeight;

        // Determine how much to scale down the image
        int scaleFactor = Math.min(photoW / targetW, photoH / targetH);

        // Decode the image file into a Bitmap sized to fill the View
        bmOptions.inJustDecodeBounds = false;
        bmOptions.inSampleSize = scaleFactor;
        bmOptions.inPurgeable = true;

        Bitmap bitmap = BitmapFactory.decodeFile(mCurrentPhotoPath, bmOptions);
        mSnippet.setImageBitmap(bitmap);
    }

    @Override
    public void onLocationChanged(Location location) {
        mLat = location.getLatitude() + "";
        mLon = location.getLongitude() + "";
    }

    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {
    }

    @Override
    public void onProviderEnabled(String provider) {
    }

    @Override
    public void onProviderDisabled(String provider) {
    }

    private class GetMunicipalities extends AsyncTask<Void, Void, String[]> {

        @Override
        protected String[] doInBackground(Void... params) {

            try {
                HttpClient client = new DefaultHttpClient();
                // Execute GETT method
                HttpGet request = new HttpGet(Consts.SERVICE_MUNICIPALITIES);
                request.setHeader("Accept", "application/json");
                request.setHeader("Content-type", "application/json");
                HttpResponse response = client.execute(request);

                // get the response
                HttpEntity responseEntity = response.getEntity();
                String jsonResponse = EntityUtils.toString(responseEntity);
                // FIXME for some reason there are extra encoded characters before the array
                jsonResponse = jsonResponse.substring(jsonResponse.indexOf("{"));
                // save the response to preferences
                Utils.setStringToPrefs(getActivity(), Consts.PREF_KEY_MUNICIPALITIES, jsonResponse);
                Utils.setLongToPrefs(getActivity(), Consts.PREF_KEY_MUNICIPALITIES_TIMESTAMP, System.currentTimeMillis());

                Municipalities municipalities = new Gson().fromJson(jsonResponse, Municipalities.class);
                String[] municipalitiesArray = new String[municipalities.getMunicipalities().size()];
                int i = 0;
                for (Municipality municipality : municipalities.getMunicipalities())
                    municipalitiesArray[i++] = municipality.getName();

                return municipalitiesArray;
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }

        @Override
        protected void onPostExecute(String[] municipalities) {
            // cannot get municipalities from server, use default values
            if (municipalities == null || municipalities.length == 0)
                return;
            if (!isAdded())
                return;
            // add the new municipalities to the list
            mMunicipality.setAdapter(new ArrayAdapter<String>(getActivity(), android.R.layout.simple_spinner_dropdown_item, municipalities));
            super.onPostExecute(municipalities);
        }
    }
}
