package com.milieukontakt.reagiram.fragments;

import android.animation.Animator;
import android.animation.ObjectAnimator;
import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AbsListView;
import android.widget.ListView;

import com.google.gson.Gson;
import com.milieukontakt.reagiram.R;
import com.milieukontakt.reagiram.ReportActivity;
import com.milieukontakt.reagiram.adapters.StatusesAdapter;
import com.milieukontakt.reagiram.models.Profile;
import com.milieukontakt.reagiram.models.StatusCheck;
import com.milieukontakt.reagiram.models.Statuses;
import com.milieukontakt.reagiram.utils.Consts;
import com.milieukontakt.reagiram.utils.Utils;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by MedoHome on 11.10.13.
 */
public class StatusesFragment extends Fragment implements View.OnClickListener {

    private View mView;
    private View mFooter;
    private ListView mList;
    private int lastFirstChild;
    private int lastY;
    private int state = Consts.ON_SCREEN;
    private ObjectAnimator hideAnimator;
    private ObjectAnimator returnAnimator;
    private String mCurrentPhotoPath;

    @SuppressLint("NewApi")
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        mView = inflater.inflate(R.layout.fragment_home, container, false);
        mFooter = mView.findViewById(R.id.footer_quick);
        mFooter.findViewById(R.id.txt_photo).setOnClickListener(this);
        mFooter.findViewById(R.id.txt_gallery).setOnClickListener(this);

        mList = (ListView) mView.findViewById(R.id.list_home);
        // do quick return for newer API version
        if (Build.VERSION.SDK_INT >= 14) {

            returnAnimator = ObjectAnimator.ofFloat(mFooter, "translationY", 0);
            returnAnimator.addListener(
                    new Animator.AnimatorListener() {
                        @Override
                        public void onAnimationStart(Animator animator) {
                            state = Consts.RETURNING;
                        }

                        @Override
                        public void onAnimationEnd(Animator animator) {
                            state = Consts.ON_SCREEN;
                        }

                        @Override
                        public void onAnimationCancel(Animator animator) {
                            state = Consts.OFF_SCREEN;
                        }

                        @Override
                        public void onAnimationRepeat(Animator animator) {
                        }
                    }
            );
            hideAnimator = ObjectAnimator.ofFloat(mFooter, "translationY", getResources().getDimension(R.dimen.footer_height));
            hideAnimator.addListener(
                    new Animator.AnimatorListener() {
                        @Override
                        public void onAnimationStart(Animator animator) {
                            state = Consts.HIDING;
                        }

                        @Override
                        public void onAnimationEnd(Animator animator) {
                            state = Consts.OFF_SCREEN;
                        }

                        @Override
                        public void onAnimationCancel(Animator animator) {
                            state = Consts.ON_SCREEN;
                        }

                        @Override
                        public void onAnimationRepeat(Animator animator) {
                        }
                    }
            );

            mList.setOnScrollListener(new AbsListView.OnScrollListener() {
                @Override
                public void onScrollStateChanged(AbsListView view, int scrollState) {

                }

                @Override
                public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount, int totalItemCount) {
                    // Computes the scroll Y position
                    final View firstChild = mList.getChildAt(firstVisibleItem);
                    int yPosition = firstChild == null ? 0 : firstChild.getTop();

                    switch (state) {
                        case Consts.OFF_SCREEN:
                            // Return quick return bar if first visible child is the same AND it's Y position increases
                            // Return quick return bar if first visible child changes to a previous sibling
                            // AND only if the quick return bar isn't already returning
                            if (!quickReturnBarIsReturning()
                                    && (firstVisibleItem == lastFirstChild && yPosition > lastY)
                                    || firstVisibleItem < lastFirstChild)
                                returnAnimator.start();
                            break;

                        case Consts.ON_SCREEN:
                            // Hide quick return bar if first visible child is the same AND it's Y position decreases
                            // Hide quick return bar if first visible child changes to a later sibling
                            // AND only if the quick return bar isn't already going away
                            if (!quickReturnBarIsGoingAway()
                                    && (firstVisibleItem == lastFirstChild && yPosition < lastY)
                                    || firstVisibleItem > lastFirstChild)
                                hideAnimator.start();
                            break;

                        case Consts.RETURNING:
                            // Cancel return of quick return bar if first visible child is the same AND it's Y position decreases
                            // Cancel return of quick return bar if first visible child changes to a later sibling
                            if ((firstVisibleItem == lastFirstChild && yPosition < lastY)
                                    || firstVisibleItem > lastFirstChild) {
                                returnAnimator.cancel();
                                hideAnimator.start();
                            }
                            break;

                        case Consts.HIDING:
                            // Cancel hide of quick return bar if first visible child is the same AND it's Y position increases
                            // Cancel hide of quick return bar if first visible child changes to a previous sibling
                            if ((firstVisibleItem == lastFirstChild && yPosition > lastY)
                                    || firstVisibleItem < lastFirstChild) {
                                hideAnimator.cancel();
                                returnAnimator.start();
                            }
                            break;
                    }
                    lastFirstChild = firstVisibleItem;
                    lastY = yPosition;
                }
            });
        }
        return mView;
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        GetReportStatuses task = new GetReportStatuses();
        task.execute();
        super.onViewCreated(view, savedInstanceState);
    }

    private void setEmptyView() {
        // create an empty card and set it as adapter
        Statuses emptyStatus = new Statuses();
        emptyStatus.setStatus("" + 0);
        List<Statuses> adapterArray = new ArrayList<Statuses>();
        adapterArray.add(emptyStatus);
        StatusesAdapter adapter = new StatusesAdapter(getActivity(), adapterArray);
        mList.addFooterView(getActivity().getLayoutInflater().inflate(R.layout.footer_trademark, null, false));
        mList.setAdapter(adapter);
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.txt_gallery:
                Intent intent = new Intent();
                intent.setType("image/*");
                intent.setAction(Intent.ACTION_GET_CONTENT);
                startActivityForResult(intent, Consts.ACTION_PICK_PHOTO);
                break;
            case R.id.txt_photo:
                if (Utils.isIntentAvailable(getActivity(), MediaStore.ACTION_IMAGE_CAPTURE)) {
                    Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
                    try {
                        File f = createImageFile();
                        takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, Uri.fromFile(f));
                        startActivityForResult(takePictureIntent, Consts.ACTION_TAKE_PHOTO);
                    } catch (Exception e) {
                        // TODO handle error
                        e.printStackTrace();
                    }
                }
                break;
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {

        if (requestCode == Consts.ACTION_PICK_PHOTO && data != null && data.getData() != null) {
            Uri _uri = data.getData();

            // image selected
            Cursor cursor = getActivity()
                    .getContentResolver()
                    .query(_uri, new String[]{android.provider.MediaStore.Images.ImageColumns.DATA}, null, null, null);
            cursor.moveToFirst();

            // get tje image path
            String imageFilePath = cursor.getString(0);
            cursor.close();

            Intent reportActivity = new Intent(getActivity(), ReportActivity.class);
            reportActivity.putExtra(Consts.EXTRA_PHOTO_PATH, imageFilePath);
            startActivity(reportActivity);
        } else if (requestCode == Consts.ACTION_TAKE_PHOTO) {
            galleryAddPic();
            Intent reportActivity = new Intent(getActivity(), ReportActivity.class);
            reportActivity.putExtra(Consts.EXTRA_PHOTO_PATH, mCurrentPhotoPath);
            startActivity(reportActivity);
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    private File createImageFile() throws IOException {
        // create album directory
        File storageDir = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES), getString(R.string.app_name));
        if (!storageDir.exists())
            storageDir.mkdirs();
        // Create an image file name
        String imageFileName = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
        File image = File.createTempFile(
                imageFileName,
                ".jpg",
                storageDir
        );
        mCurrentPhotoPath = image.getAbsolutePath();
        return image;
    }

    private void galleryAddPic() {
        Intent mediaScanIntent = new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE);
        File f = new File(mCurrentPhotoPath);
        Uri contentUri = Uri.fromFile(f);
        mediaScanIntent.setData(contentUri);
        getActivity().sendBroadcast(mediaScanIntent);
    }

    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
    private boolean quickReturnBarIsReturning() {
        return returnAnimator.isRunning() || returnAnimator.isStarted();
    }

    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
    private boolean quickReturnBarIsGoingAway() {
        return hideAnimator.isRunning() || hideAnimator.isStarted();
    }

    private class GetReportStatuses extends AsyncTask<Void, Void, StatusCheck> {

        @Override
        protected void onPreExecute() {
            if (!isAdded())
                return;
            getActivity().setProgressBarIndeterminateVisibility(true);
        }

        @Override
        protected StatusCheck doInBackground(Void... params) {

            try {
                HttpClient client = new DefaultHttpClient();
                // Execute GET method
                HttpGet request = new HttpGet(Consts.SERVICE_REPORTS);
                request.setHeader("Accept", "application/json");
                request.setHeader("Content-type", "application/json");
                HttpResponse response = client.execute(request);

                // get the response
                HttpEntity responseEntity = response.getEntity();
                String jsonResponse = EntityUtils.toString(responseEntity);
                StatusCheck statuses = new Gson().fromJson(jsonResponse, StatusCheck.class);

                return statuses;
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }

        @Override
        protected void onPostExecute(StatusCheck statusCheck) {
            if (!isAdded())
                return;
            // hide the progress bar
            getActivity().setProgressBarIndeterminateVisibility(false);

            if (statusCheck == null || statusCheck.getStatuses().isEmpty()) {
                String jsonReport = Utils.getStringFromPrefs(getActivity(), Consts.PREF_KEY_REPORTS);
                if (Utils.isConnected(getActivity()) || jsonReport == null)
                    setEmptyView();
                else {
                    statusCheck = new Gson().fromJson(jsonReport, StatusCheck.class);
                    StatusesAdapter adapter = new StatusesAdapter(getActivity(), statusCheck.getStatuses());
                    mList.addFooterView(getActivity().getLayoutInflater().inflate(R.layout.footer_trademark, null, false));
                    mList.setAdapter(adapter);
                }
                return;
            }
            List<Statuses> adapterList = statusCheck.getStatuses();
            // check if user has an ID and sort the list accordingly
            String jsonProfile = Utils.getStringFromPrefs(getActivity(), Consts.PREF_KEY_PROFILE);
            if (jsonProfile != null) {
                Profile profile = new Gson().fromJson(jsonProfile, Profile.class);
                String reporterId = profile.getId();

                // separate the lists by user reported or reported by others
                List<Statuses> userList = new ArrayList<Statuses>();
                List<Statuses> otherList = new ArrayList<Statuses>();
                for (Statuses status : adapterList)
                    if (status.getUserId().equalsIgnoreCase(reporterId))
                        userList.add(status);
                    else
                        otherList.add(status);
                // join the two lists so we can make the adapter
                adapterList = new ArrayList<Statuses>();
                adapterList.addAll(userList);
                adapterList.addAll(otherList);
                // save the number of user reports
                Utils.setLongToPrefs(getActivity(), Consts.PREF_KEY_REPORTS_NUMBER, userList.size());
            }
            // save to preferences
            Utils.setStringToPrefs(getActivity(), Consts.PREF_KEY_REPORTS, new Gson().toJson(statusCheck, StatusCheck.class));
            // set list adapter
            StatusesAdapter adapter = new StatusesAdapter(getActivity(), adapterList);
            mList.addFooterView(getActivity().getLayoutInflater().inflate(R.layout.footer_trademark, null, false));
            mList.setAdapter(adapter);

            super.onPostExecute(statusCheck);
        }
    }
}
