package com.milieukontakt.reagiram.services;

import android.app.IntentService;
import android.app.NotificationManager;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.support.v4.app.NotificationCompat;
import android.util.Base64;

import com.google.gson.Gson;
import com.milieukontakt.reagiram.R;
import com.milieukontakt.reagiram.models.Report;
import com.milieukontakt.reagiram.models.ReportQueue;
import com.milieukontakt.reagiram.models.ReportRequest;
import com.milieukontakt.reagiram.models.ServerResponse;
import com.milieukontakt.reagiram.utils.Consts;
import com.milieukontakt.reagiram.utils.Utils;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicHeader;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.List;

public class ReportService extends IntentService {

    NotificationManager mNotifyManager;

    public ReportService() {
        super("ReportService");
    }

    public ReportService(String name) {
        super(name);
    }

    @Override
    protected void onHandleIntent(Intent intent) {
        String jsonQueue = Utils.getStringFromPrefs(getApplicationContext(), Consts.PREF_KEY_QUEUE);
        if (jsonQueue == null)
            return;
        // create progress notification
        NotificationCompat.Builder mBuilder =
                new NotificationCompat.Builder(this)
                        .setSmallIcon(android.R.drawable.stat_sys_upload)
                        .setContentTitle(getString(R.string.app_name))
                        .setContentText(getString(R.string.toast_uploading))
                        .setProgress(0, 0, true)
                        .setAutoCancel(true)
                        .setTicker(getString(R.string.toast_uploading))
                        .setOngoing(true);
        // Issues the notification
        mNotifyManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
        mNotifyManager.notify(0, mBuilder.build());

        Gson gson = new Gson();
        ReportQueue queue = gson.fromJson(jsonQueue, ReportQueue.class);
        List<Report> newQueue = new ArrayList<Report>();

        int i = -1;
        for (Report report : queue.getReports()) {
            i++;
            ReportRequest request = new ReportRequest();
            // set user id and fill in the rest of the report
            request.setUserId(report.getProfile().getId());
            request.setName(report.getProfile().getFirstName());
            request.setLastName(report.getProfile().getLastName());
            request.setEmail(report.getProfile().getEmail());
            request.setAge(report.getProfile().getAge());
            request.setGender(report.getProfile().getGender());
            request.setMunicipality(report.getMunicipality());
            request.setLocation(report.getLocation());
            request.setDescription(report.getDescription());
            request.setCategory(report.getCategory());
            request.setLatitude(report.getLatitude());
            request.setLongitude(report.getLongitude());
            request.setOperator(report.getOperator());
            request.setTime(report.getTime());
            request.setBase64EncodedString(encodeAndScaleImage(report.getImageUrl()));

            try {
                HttpClient client = new DefaultHttpClient();
                // Execute POST method
                HttpPost postRequest = new HttpPost(Consts.SERVICE_UPLOAD);
                StringEntity se = new StringEntity(gson.toJson(request, ReportRequest.class), "UTF-8");
                se.setContentType(new BasicHeader(HTTP.CONTENT_TYPE, "application/json"));
                postRequest.setEntity(se);
                HttpResponse response = client.execute(postRequest);

                // get the response
                HttpEntity responseEntity = response.getEntity();
                String jsonResponse = EntityUtils.toString(responseEntity);

                // check the status
                ServerResponse serverResponse = gson.fromJson(jsonResponse, ServerResponse.class);
                if (serverResponse.getStatus().intValue() <= 0)
                    newQueue.add(report);
                else {
                    // count +1 report
                    long reportsNumber = Utils.getLongFromPrefs(getApplicationContext(), Consts.PREF_KEY_REPORTS_NUMBER);
                    Utils.setLongToPrefs(getApplicationContext(), Consts.PREF_KEY_REPORTS_NUMBER, reportsNumber + 1);
                }
            } catch (Exception e) {
                e.printStackTrace();
                newQueue.add(report);
                continue;
            }
        }

        if (newQueue.size() == 0)
            // all done
            Utils.setStringToPrefs(getApplicationContext(), Consts.PREF_KEY_QUEUE, null);
        else {
            // save the new queue for later in case of failure
            queue.setReports(newQueue);
            Utils.setStringToPrefs(getApplicationContext(), Consts.PREF_KEY_QUEUE, gson.toJson(queue, ReportQueue.class));
        }
        mNotifyManager.cancel(0);
    }

    private String encodeAndScaleImage(String imagePath) {

        Bitmap original = BitmapFactory.decodeFile(imagePath);
        if (original == null)
            return null;

        // the smallest size has to be constant
        // the other size should scale accordingly
        int width = original.getWidth();
        int height = original.getHeight();
        if (width <= height) {
            float ratio = (float) height / (float) width;
            width = Consts.THUMB_MIN_SIZE;
            height = (int) (width * ratio);
        } else {
            float ratio = (float) height / (float) width;
            height = Consts.THUMB_MIN_SIZE;
            width = (int) (height * ratio);
        }

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        // if image is larger than 1024px X 1024px (by longest side)
        if (!(original.getWidth() < Consts.THUMB_MIN_SIZE || original.getHeight() < Consts.THUMB_MIN_SIZE)) {
            // resize picture to 1024px X 1024px (by longest side)
            Bitmap scaledBitmap = Bitmap.createScaledBitmap(original, width, height, true);
            scaledBitmap.compress(Bitmap.CompressFormat.JPEG, 90, baos);
        } else
            original.compress(Bitmap.CompressFormat.JPEG, 90, baos);
        
        byte[] byteArrayImage = baos.toByteArray();
        return Base64.encodeToString(byteArrayImage, Base64.DEFAULT);
    }
}
